package uk.ac.leeds.ccg.grids;
/**
 * This file is distributed as part of the uk.ac.leeds.ccg.grids library 
 * This library is a resource for manipulating spatial data stored as grids.
 * Copyright (C) 2002  Andy Turner
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
import uk.ac.leeds.ccg.grids.Utilities;
import uk.ac.leeds.ccg.raster.Raster;
//import uk.ac.leeds.ccg.geotools.GeoRectangle;
import java.io.File;
import java.io.RandomAccessFile;
/**
 * Last modified 14 Nov 2002
 * @author  andyt
 * @version 1.0.1
 * @description
 */
public class Grid2DSquareCellDoubleFileFactory extends AbstractGrid2DSquareCellDoubleFactory {
    
    /**
     * For storing data directory
     */
    String dataDirectory = null;

    /**
     * For storing file
     */
    File file = null;

    /** Creates new Grid2DSquareCellDoubleFileFactory. */
    public Grid2DSquareCellDoubleFileFactory() {
        this( System.getProperty( "java.io.tmpdir" ) );
    }
    public Grid2DSquareCellDoubleFileFactory( String dataDirectory ) {
        setDataDirectory( dataDirectory );
    }
    public Grid2DSquareCellDoubleFileFactory( File file ) {
        setFile( file );
        setDataDirectory( file.getParent() );
    }
    
    public AbstractGrid2DSquareCellDouble createGrid2DSquareCellDouble( int nrows, int ncols, double xllcorner, double yllcorner, double cellsize, double noDataValue, int gridStatisticsType ) {
        return new Grid2DSquareCellDoubleFile( doFile(), nrows, ncols, xllcorner, yllcorner, cellsize, noDataValue, gridStatisticsType );
    }
    
    public AbstractGrid2DSquareCellDouble createGrid2DSquareCellDouble( AbstractGrid2DSquareCellDouble grid, int startRowIndex, int startColIndex, int endRowIndex, int endColIndex, int gridStatisticsType ) {
        // Default dataDirectory to the same directory as grid if it is stored in a file
        if ( grid instanceof Grid2DSquareCellDoubleFile ) {
            setDataDirectory( ( ( Grid2DSquareCellDoubleFile ) grid ).getDataDirectory() );
        }
        return new Grid2DSquareCellDoubleFile( doFile(), grid, startRowIndex, startColIndex, endRowIndex, endColIndex, gridStatisticsType );
    }
    
    public AbstractGrid2DSquareCellDouble createGrid2DSquareCellDouble( File gridFile, int startRowIndex, int startColIndex, int endRowIndex, int endColIndex, int gridStatisticsType ) {
        return new Grid2DSquareCellDoubleFile( doFile(), gridFile, startRowIndex, startColIndex, endRowIndex, endColIndex, gridStatisticsType );
    }
    
    public AbstractGrid2DSquareCellDouble createGrid2DSquareCellDouble( Raster raster, int gridStatisticsType ) {
        return new Grid2DSquareCellDoubleFile( doFile(), raster, gridStatisticsType );
    }
    
    /**
     * Returns dataDirectory
     */
    public String getDataDirectory() {
        return this.dataDirectory;
    }
    
    /**
     * Sets dataDirectory
     */
    public void setDataDirectory( String dataDirectory ) {
        this.dataDirectory = dataDirectory;
    }
    
    /**
     * Returns file
     */
    public File getFile() {
        return this.file;
    }
    
    /**
     * Sets file
     */
    public void setFile( File file ) {
        this.file = file;
        setDataDirectory( file.getParent() + file.separator );
    }
    
    /**
     * Returns the file for the grid
     */
    private File doFile() {
        File file = getFile();
        if ( file == null ) {
        //if ( ! file.exists() ) {
            setFile( makeTemporaryFile() );
            return getFile();
        }
        return file;
    }
    
    /**
     * Returns a temporary File  
     */
    protected File makeTemporaryFile() {
        return makeTemporaryFile( "f" );
    }
    protected File makeTemporaryFile( String firstPartOfFilename ) {
        Utilities u = new Utilities();
        return u.makeTemporaryFile( getDataDirectory(), firstPartOfFilename );
    }
    
    public String toString() {
        return "Grid2DSquareCellDoubleFileFactory( " + getFile().toString() + " )";
    }

}
